@@ -1,11 +1,14 @@
 package Sepia::Debug;
 # use Sepia;
-require Carp;
+use Carp ();                    # old Carp doesn't export shortmess.
 use Text::Abbrev;
 use strict;
 use vars qw($pack $file $line $sub $level
             $STOPDIE $STOPWARN);
 
+sub define_shortcut;
+*define_shortcut = *Sepia::define_shortcut;
+
 BEGIN {
     ## Just leave it on -- with $DB::trace = 0, there doesn't seem
     ## to be a perforamnce penalty!
@@ -23,7 +26,6 @@ BEGIN {
 sub repl_debug
 {
     debug(@_);
-    0;
 }
 
 sub repl_backtrace
@@ -33,13 +35,17 @@ sub repl_backtrace
         last unless $pack;
         print($i == $level+3 ? "*" : ' ', " [$i]\t$sub ($file:$line)\n");
     }
-    0
 }
 
 # return value from die
 sub repl_return
 {
-    (1, $Sepia::REPL{eval}->(@_));
+    if ($Sepia::WANTARRAY) {
+        @Sepia::REPL_RESULT = $Sepia::REPL{eval}->(@_);
+    } else {
+        $Sepia::REPL_RESULT[0] = $Sepia::REPL{eval}->(@_);
+    }
+    last repl;
 }
 
 sub repl_lsbreak
@@ -78,35 +84,6 @@ sub tie_class
                 : die "Sorry, can't tie $sig\n";
 }
 
-# {
-#     require Tie::Array;
-#     require Tie::Hash;
-#     require Tie::Scalar;
-#     package Sepia::Array;
-#     our @ISA = qw(Tie::StdArray);
-#     sub TIEARRAY { bless $_[1], $_[0] }
-#     package Sepia::Hash;
-#     our @ISA = qw(Tie::StdHash);
-#     sub TIEHASH { bless $_[1], $_[0] }
-#     package Sepia::Scalar;
-#     our @ISA = qw(Tie::StdScalar);
-#     sub TIESCALAR { bless $_[1], $_[0] }
-# }
-
-# sub eval_in_env3
-# {
-#     my ($expr, $env) = @_;
-#     my @vars = grep /^([\$\@%])(.+)/, keys %$env;
-#     my $body = 'sub { my ('.join(',', @vars).');';
-#     for my $i (0..$#vars) {
-#         $body .= "tie $vars[$i], ".tie_class($vars[$i]).', $_['.$i.'];';
-#     }
-#     $body .= "$expr }";
-#     print STDERR "---\n$body\n---\n";
-#     $body = eval $body;
-#     $@ || $body->(@{$env}{@vars});
-# }
-
 ## XXX: this is a better approach (the local/tie business is vile),
 ## but it segfaults and I'm not sure why.
 sub eval_in_env2
@@ -127,12 +104,7 @@ sub eval_in_env2
 # evaluate EXP LEV levels up the stack
 sub repl_upeval
 {
-    my $exp = shift;
-    # my ($lev, $exp) = $_[0] =~ /^\s*(\d+)\s+(.*)/;
-    # print " <= $exp\n";
-    # (0, eval_in_env2($exp, $level));
-    # (0, eval_in_env3($exp, peek_my(4 + $level)));
-    eval_in_env($exp, peek_my(4+$level));
+    eval_in_env(shift, peek_my(4+$level));
 }
 
 # inspect lexicals at level N, or current level
@@ -150,10 +122,9 @@ sub repl_inspect
         print "[$i] $sub:\n";
         for (sort keys %$h) {
             local @Sepia::res = $h->{$_};
-            print "\t$_ = ", $Sepia::PRINTER->(), "\n";
+            print "\t$_ = ", $Sepia::PRINTER{$Sepia::PRINTER}->(), "\n";
         }
     }
-    0;
 }
 
 sub debug
@@ -199,12 +170,13 @@ sub repl_break
     $arg =~ s/^\s+//;
     $arg =~ s/\s+$//;
     my ($f, $l, $cond) = $arg =~ /^(.+?):(\d+)\s*(.*)/;
-    $cond ||= 1;
+    $cond = 1 unless $cond =~ /\S/;
     $f ||= $file;
     $l ||= $line;
-    print "break ", breakpoint($f, $l, $cond), "\n";
-    0;
-}
+    return unless defined $f && defined $l;
+    my $bp = breakpoint($f, $l, $cond);
+    print "break $bp\n" if $bp;
+ }
 
 sub update_location
 {
@@ -222,7 +194,6 @@ sub repl_list
     my @lines = eval shift;
     @lines = $line - 5 .. $line + 5 unless @lines;
     printf '%-6d%s', $_, ${"::_<$file"}[$_-1] for @lines;
-    0
 }
 
 sub repl_delete
@@ -232,113 +203,71 @@ sub repl_delete
     $l ||= $line;
     my $h = breakpoint_file $f;
     delete $h->{$l} if defined $h;
-    0
 }
 
-my %parent_repl = (
-    delete => \&repl_delete,
-    debug => \&repl_debug,
-    break => \&repl_break,
-    lsbreak => \&repl_lsbreak,
-);
-
-my %parent_doc = (
-    break =>
-        'break [F:N [E]]    Set a breakpoint in F at line N (or at current
-                       position), enabled if E evalutes to true.',
-    delete =>
-        'delete             Delete current breakpoint.',
-    debug =>
-        'debug [0|1]        Enable or disable debugging.',
-    lsbreak =>
-        'lsbreak            List breakpoints.',
-);
-
 sub add_repl_commands
 {
-    %Sepia::REPL = (%Sepia::REPL, %parent_repl);
-    %Sepia::REPL_DOC = (%Sepia::REPL_DOC, %parent_doc);
+    define_shortcut 'delete', \&repl_delete,
+        'Delete current breakpoint.';
+    define_shortcut 'debug', \&repl_debug,
+        'debug [0|1]', 'Enable or disable debugging.';
+    define_shortcut 'break', \&repl_break,
+        'break [F:N [E]]',
+        'Set a breakpoint in F at line N (or at current position), enabled if E evalutes to true.';
+    define_shortcut 'lsbreak', \&repl_lsbreak,
+        'List breakpoints.';
     %Sepia::RK = abbrev keys %Sepia::REPL;
 }
 
-my %REPL = (
-    up => sub {
+sub add_debug_repl_commands
+{
+
+    define_shortcut up => sub {
         $level += shift || 1;
         update_location(4);
         show_location;
-        0
-    },
-    down => sub {
+    }, 'up [N]', 'Move up N stack frames.';
+    define_shortcut down => sub {
         $level -= shift || 1;
         $level = 0 if $level < 0;
         update_location(4);
         show_location;
-        0
-    },
-
-    continue => sub {
+    }, 'down [N]', 'Move down N stack frames.';
+    define_shortcut continue => sub {
         $level = 0;
-        $DB::single = 0; 1
-    },
+        $DB::single = 0;
+        last repl;
+    }, 'Yep.';
 
-    next => sub {
+    define_shortcut next => sub {
         my $n = shift || 1;
         $DB::single = 0;
-        breakpoint $file, $line + $n, 'next'; 1
-    },
+        breakpoint $file, $line + $n, 'next';
+        last repl;
+    }, 'next [N]', 'Advance N lines, skipping subroutines.';
 
-    step => sub {
-        $DB::single = shift || 1; 1
-    },
+    define_shortcut step => sub {
+        $DB::single = shift || 1;
+        last repl;
+    }, 'step [N]', 'Step N lines forward, entering subroutines.';
 
-    break => \&repl_break,
-
-    list => \&repl_list,
-
-    # quit => sub {
-    #     debug(0);
-    # },
-    backtrace => \&repl_backtrace,
-    inspect => \&repl_inspect,
-    # eval => \&repl_upeval,
-    return => \&repl_return,
-    lsbreak => \&repl_lsbreak,
-    eval => \&repl_upeval,      # DANGER!
-);
-
-my %REPL_DOC = (
-    continue =>
-        'continue        Yep.',
-    next =>
-        'next [N]        Advance N lines, skipping subroutines.',
-    list =>
-        'list EXPR       List source lines of current file.',
-    step =>
-        'step [N]        Step N lines forward, entering subroutines.',
-    quit =>
-        'quit            Exit the current prompt level.',
-    up =>
-        'up [N]          Move up N stack frames.',
-    down =>
-        'down [N]        Move down N stack frames.',
-    backtrace =>
-        'backtrace       show backtrace',
-    inspect =>
-        'inspect [N]     inspect lexicals in frame N (or current)',
-    eval =>
-        'eval EXPR       evaluate EXPR in current frame',
-    return =>
-        'return EXPR     return EXPR',
-    quit =>
-        'quit            keep on dying/warning',
- );
+    define_shortcut list => \&repl_list,
+        'list EXPR', 'List source lines of current file.';
+    define_shortcut backtrace => \&repl_backtrace, 'show backtrace';
+    define_shortcut inspect => \&repl_inspect,
+        'inspect [N]', 'inspect lexicals in frame N (or current)';
+    define_shortcut return => \&repl_return, 'return EXPR', 'return EXPR';
+    define_shortcut eval => \&repl_upeval,
+        'eval EXPR', 'evaluate EXPR in current frame';      # DANGER!
+}
 
 sub repl
 {
     show_location;
-
-    local %Sepia::REPL = (%Sepia::REPL, %REPL, @_);
-    local %Sepia::REPL_DOC = (%Sepia::REPL_DOC, %REPL_DOC);
+    local %Sepia::REPL = %Sepia::REPL;
+    local %Sepia::REPL_DOC = %Sepia::REPL_DOC;
+    add_debug_repl_commands;
+    map { define_shortcut @$_ } @_;
     local %Sepia::RK = abbrev keys %Sepia::REPL;
     # local $Sepia::REPL_LEVEL = $Sepia::REPL_LEVEL + 1;
     local $Sepia::PS1 = "*$Sepia::REPL_LEVEL*> ";
@@ -360,7 +289,7 @@ sub DB::DB
         if ($cond eq 'next') {
             delete $main::{"_<$file"}{$line};
         } else {
-            return unless eval $cond;
+            return unless $Sepia::REPL{eval}->($cond);
         }
     }
     repl();
@@ -375,16 +304,21 @@ sub die
         my @dieargs = @_;
         local $level = 0;
         local ($pack, $file, $line, $sub) = caller($level);
-        print "@_\n\tin $sub\nDied $MSG\n";
+        my $tmp = "@_";
+        $tmp .= "\n" unless $tmp =~ /\n\z/;
+        print "$tmp\tin $sub\nDied $MSG\n";
         my $trace = $DB::trace;
         $DB::trace = 1;
         repl(
-            die => sub { local $STOPDIE=0; CORE::die @dieargs },
-            quit => sub { local $STOPDIE=0; CORE::die @dieargs });
+            [die => sub { local $STOPDIE=0; CORE::die @dieargs },
+             'Continue dying.'],
+            [quit => sub { local $STOPDIE=0; CORE::die @dieargs },
+             'Continue dying.']);
         $DB::trace = $trace;
     } else {
         CORE::die(Carp::shortmess @_);
     }
+    1;
 }
 
 sub warn
@@ -398,8 +332,10 @@ sub warn
         local ($pack, $file, $line, $sub) = caller($level);
         print "@_\n\tin $sub\nWarned $MSG\n";
         repl(
-            warn => sub { local $STOPWARN=0; CORE::warn @dieargs },
-            quit => sub { local $STOPWARN=0; CORE::warn @dieargs });
+            [warn => sub { local $STOPWARN=0; CORE::warn @dieargs },
+             'Continue warning.'],
+            [quit => sub { local $STOPWARN=0; CORE::warn @dieargs },
+             'Continue warning.']);
         $DB::trace = $trace;
     } else {
         ## Avoid showing up in location information.
